
/*
 Fichier permettant de creer une geometrie 2D de coupleur
          magnetique coaxial quelconque.
      Julien Fontchastagner, UL-GREEN, mars 2019.
*/

// Donnes geometriques d'entree et dialogue avec GetDP
Include "data.geo";

// On cree la geometrie a l'ancienne
SetFactory("Built-in");
Geometry.ExtrudeReturnLateralEntities = 1;

// Points
Point(1) = {0,0,0};
Point(2) = {R0,0,0};
Point(3) = {R1,0,0};
Point(4) = {R2,0,0};
Point(5) = {R3,0,0};
Point(6) = {R4,0,0};
Point(7) = {R5,0,0};

// Lignes
Line(1) = {2,3};
Line(2) = {3,4};
Line(3) = {4,5};
Line(4) = {5,6};
Line(5) = {6,7};
Transfinite Line{1,2,3,4,5} = ndiv+1 Using Progression 1.0;
ldroite[] = {1,2,3,4,5};

// Differents cas geometriques possibles selon 
//   les largeurs angulaires des aimants
If ( alpha2 < (1.0-alpha1) )
  cas = 2;
EndIf
If ( alpha2 > (1.0-alpha1) )
  cas = 3;
EndIf
If (Fabs(1.0-alpha1-alpha2) <= 1e-2*alpha2) // egalite ou presque
  cas = 1;
  alpha2 = 1.0-alpha1;
EndIf

// Construction des Faces par extrusion rotative
scul1[] = {};
saim1[] = {};
sair1[] = {};
sentrf[] = {};
saim2n[] = {};
saim2s[] = {};
sair2[] = {};
scul2[] = {};
ldir[] = {};

// 1er cas : alpha1+alpha2 = 1
If (cas == 1)
  l_to_extrude[] = ldroite[];
  nlayers = R3*alpha2/2*Pi/pp/lmailg;
    If (nlayers < 2.0)
    nlayers = 2;
  EndIf
  outs[] = Extrude { {0,0,1}, {0,0,0}, alpha2/2*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers}; Recombine; };
  scul1[] += {outs[1]};
  sair1[] += {outs[5]};
  sentrf[] += {outs[9]};
  saim2n[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  l_to_extrude[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};
  nlayers2 = R3*alpha1*Pi/pp/lmailg;
    If (nlayers2 < 2.0)
    nlayers2 = 2;
  EndIf
  outs[] = Extrude { {0,0,1}, {0,0,0}, alpha1*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers2}; Recombine; };
  scul1[] += {outs[1]};
  saim1[] += {outs[5]};
  sentrf[] += {outs[9]};
  sair2[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  l_to_extrude[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};
  outs[] = Extrude { {0,0,1}, {0,0,0}, alpha2/2*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers}; Recombine; };
  scul1[] += {outs[1]};
  sair1[] += {outs[5]};
  sentrf[] += {outs[9]};
  saim2s[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  lgauche[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};
EndIf

// 2eme cas : alpha1+alpha2 < 1
If (cas == 2)
  l_to_extrude[] = ldroite[];
  nlayers = R3*alpha2/2*Pi/pp/lmailg;
    If (nlayers < 2.0)
    nlayers = 2;
  EndIf
  outs[] = Extrude { {0,0,1}, {0,0,0}, alpha2/2*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers}; Recombine; };
  scul1[] += {outs[1]};
  sair1[] += {outs[5]};
  sentrf[] += {outs[9]};
  saim2n[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  l_to_extrude[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};
  nlayers2 = R3*(1.0-alpha1-alpha2)/2*Pi/pp/lmailg;
    If (nlayers2 < 2.0)
    nlayers2 = 2;
  EndIf
  outs[] = Extrude { {0,0,1}, {0,0,0}, (1.0-alpha1-alpha2)/2*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers2}; Recombine; };
  scul1[] += {outs[1]};
  sair1[] += {outs[5]};
  sentrf[] += {outs[9]};
  sair2[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  l_to_extrude[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};
  nlayers3 = R3*alpha1*Pi/pp/lmailg;
  If (nlayers3 < 2.0)
    nlayers3 = 2;
  EndIf
  outs[] = Extrude { {0,0,1}, {0,0,0}, alpha1*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers3}; Recombine; };
  scul1[] += {outs[1]};
  saim1[] += {outs[5]};
  sentrf[] += {outs[9]};
  sair2[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  l_to_extrude[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};
  outs[] = Extrude { {0,0,1}, {0,0,0}, (1.0-alpha1-alpha2)/2*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers2}; Recombine; };
  scul1[] += {outs[1]};
  sair1[] += {outs[5]};
  sentrf[] += {outs[9]};
  sair2[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  l_to_extrude[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};
  outs[] = Extrude { {0,0,1}, {0,0,0}, alpha2/2*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers}; Recombine; };
  scul1[] += {outs[1]};
  sair1[] += {outs[5]};
  sentrf[] += {outs[9]};
  saim2s[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  lgauche[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};  
EndIf

// 3eme cas : alpha1+alpha2 > 1
If (cas == 3)
  l_to_extrude[] = ldroite[];
  nlayers = R3*(1.0-alpha1)/2*Pi/pp/lmailg;
  If (nlayers < 2.0)
    nlayers = 2;
  EndIf
  outs[] = Extrude { {0,0,1}, {0,0,0}, (1.0-alpha1)/2*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers}; Recombine; };
  scul1[] += {outs[1]};
  sair1[] += {outs[5]};
  sentrf[] += {outs[9]};
  saim2n[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  l_to_extrude[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};
  nlayers2 = R3*(alpha1+alpha2-1.0)/2*Pi/pp/lmailg;
  If (nlayers2 < 2.0)
    nlayers2 = 2;
  EndIf
  outs[] = Extrude { {0,0,1}, {0,0,0}, (alpha1+alpha2-1.0)/2*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers2}; Recombine; };
  scul1[] += {outs[1]};
  saim1[] += {outs[5]};
  sentrf[] += {outs[9]};
  saim2n[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  l_to_extrude[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};
  nlayers3 = R3*(1.0-alpha2)*Pi/pp/lmailg;
  If (nlayers3 < 2.0)
    nlayers3 = 2;
  EndIf
  outs[] = Extrude { {0,0,1}, {0,0,0}, (1.0-alpha2)*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers3}; Recombine; };
  scul1[] += {outs[1]};
  saim1[] += {outs[5]};
  sentrf[] += {outs[9]};
  sair2[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  l_to_extrude[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};
  outs[] = Extrude { {0,0,1}, {0,0,0}, (alpha1+alpha2-1.0)/2*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers2}; Recombine; };
  scul1[] += {outs[1]};
  saim1[] += {outs[5]};
  sentrf[] += {outs[9]};
  saim2s[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  l_to_extrude[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};
  outs[] = Extrude { {0,0,1}, {0,0,0}, (1.0-alpha1)/2*Pi/pp} 
                   { Line {l_to_extrude[]}; Layers{nlayers}; Recombine; };
  scul1[] += {outs[1]};
  sair1[] += {outs[5]};
  sentrf[] += {outs[9]};
  saim2s[] += {outs[13]};
  scul2[] += {outs[17]};
  ldir[] += {outs[3]};
  ldir[] += {outs[18]};
  lgauche[] = {outs[0],outs[4],outs[8],outs[12],outs[16]};  
EndIf

// Regions physiques pour GetDP
// Lignes
Physical Line(DIRICHLET) = {ldir[]};
Physical Line(GAUCHE) = {lgauche[]};
Physical Line(DROITE) = {ldroite[]};
// Faces
Physical Surface(CUL1) = {scul1[]};
Physical Surface(CUL2) = {scul2[]};
Physical Surface(AIM1) = {saim1[]};
Physical Surface(AIM2N) = {saim2n[]};
Physical Surface(AIM2S) = {saim2s[]};
Physical Surface(ENTRF) = {sentrf[]};
Physical Surface(AIR) = {sair1[],sair2[]};

// Un peu de cosmetisme pour finir...
Color Gray {Surface{scul1[],scul2[]};}
Color White {Surface{sentrf[],sair1[],sair2[]};}
Color Red {Surface{saim1[]};}
Color Blue {Surface{saim2n[],saim2s[]};}


